package GrapherGUI;


import java.awt.*;
import javax.swing.*;


/** Container object for ParameterPanels.  Contains a method to easily access
 * the current value of every parameter.
 * @see ParameterPanel
 */
public class SliderMatrix {

    /** The number of initialized parameters
     */    
    int _size = 0;

    /** maximum capacity
     */        
    int _capacity = 0;
    // list of parameters
    double[] _min, _max, _start;
    int[] _numTicks;
    String[] _imageFile, _name;
    ParameterPanel[] _panelArray;

    /** Constructor
     * @param capacity maximum number of parameters this object will hold
     */        
    public SliderMatrix(int capacity) {
        _capacity = capacity;
        _min = new double[_capacity];
        _max = new double[_capacity];
        _start = new double[_capacity];
        _numTicks = new int[_capacity];
        _imageFile = new String[_capacity];
        _name = new String[_capacity];
        _panelArray = new ParameterPanel[_capacity];
    }

    /** Add a parameter to the array.
     * @param min minimum value the parameter can take
     * @param max maximum value the parameter can take
     * @param start starting value of the parameter
     * @param numticks number of marks on the slider
     * @param imagefile the JPEF or GIF image of this parameter.  If none, set to null.
     * @param name string identifier for this parameter
     * @return true if parameter added successfully
     */        
    public boolean add(double min, double max, double start, int numticks, String imagefile, String name) {
        if (_size >= _capacity) {
            // error
            return false;
        }

        _min[_size] = min;
        _max[_size] = max;
        _start[_size] = start;
        _numTicks[_size] = numticks;
        _imageFile[_size] = imagefile;
        _name[_size] = name;
        _panelArray[_size] = new ParameterPanel(_min[_size], _max[_size],
                _start[_size], _numTicks[_size], _imageFile[_size], _name[_size]);

        _size++;
        return true;
    }

    /** Creates a JPanel of ParameterPanels.
     * @return JPanel
     */        
    public JPanel[] getAllSliders() {
        JPanel[] slist = new JPanel[_size];

        for (int i = 0; i < _size; i++) {
            slist[i] = _panelArray[i];
        }
        return slist;
    }

    /** Returns an array of pointers to the JSlider objects in the ParameterPanels.
     * @return array of JSlider objects
     * @see JSlider
     */        
    public JSlider[] getJSliderArray() {
        // used in GUI.java to add changelisteners.
        JSlider[] slist = new JSlider[_size];

        for (int i = 0; i < _size; i++) {
            slist[i] = _panelArray[i].slider();
        }
        return slist;
    }

    /** Current number of parameters entered.
     * @return size
     */        
    public int size() {
        return _size;
    }

    /** Maximum capacity of this object.
     * @return capacity
     */        
    public int capacity() {
        return _capacity;
    }

    public ParameterPanel get(String param) {
        for (int i = 0; i < _size; i++) {
            if (param.equals(_panelArray[i].name())) {
                return _panelArray[i];
            }
        }
        return null;
    }

    /** Return the value of a certain parameter.
     * @param param string identifier (entered as <b>name</b> in method <b>add()</b>
     * @return current value of the parameter
     * @throws Error If a non-existing parameter is requested, this method will throw and instance
     * of Error.
     * @see Error
     */        
    public double val(String param) throws Error {
        ParameterPanel p = get(param);

        if (p == null) {
            throw (new Error("Object not found."));  
        } else {
            return p.val();
        }
    }

    public double min(String param) throws Error {
        ParameterPanel p = get(param);

        if (p == null) {
            throw (new Error("Object not found."));  
        } else {
            return p.min();
        }
    }

    public double max(String param) throws Error {
        ParameterPanel p = get(param);

        if (p == null) {
            throw (new Error("Object not found."));  
        } else {
            return p.max();
        }
    }

    public boolean incr(String param, double ds) throws Error {
        ParameterPanel p = get(param);

        if (p == null) {
            throw (new Error("Object not found."));
        } else {
            return p.incr(ds);
        }
    }

    public void snapLeft(String param) {
        ParameterPanel p = get(param);

        if (p == null) {
            return;
        } else {
            p.snapLeft();
        }
    }
}

